@@ -40,6 +40,26 @@ class CalcWidthTest(unittest.TestCase):
 		self.wtest("wide char", '\xe6\x9b\xbf', 2)
 		self.wtest("invalid", '\xe6', 1)
 
+class ConvertDecSpecialTest(unittest.TestCase):
+	def ctest(self, desc, s, exp, expcs):
+		urwid.set_encoding('ascii')
+		c = urwid.Text(s).render((5,))
+		result = c.text[0]
+		assert result==exp, desc+" got:"+`result`+" expected:"+`exp`
+		resultcs = c.cs[0]
+		assert resultcs==expcs, desc+" got:"+`resultcs`+" expected:"+`expcs`
+		
+
+	def test1(self):
+		self.ctest("no conversion", "hello", "hello", [(None,5)])
+		self.ctest("only special", u"£££££", "}}}}}", [("0",5)])
+		self.ctest("mix left", u"££abc", "}}abc", [("0",2),(None,3)])
+		self.ctest("mix right", u"abc££", "abc}}", [(None,3),("0",2)])
+		self.ctest("mix inner", u"a££bc", "a}}bc", 
+			[(None,1),("0",2),(None,2)] )
+		self.ctest("mix well", u"£a£b£", "}a}b}",
+			[("0",1),(None,1),("0",1),(None,1),("0",1)] )
+		
 
 class WithinDoubleByteTest(unittest.TestCase):
 	def setUp(self):
@@ -1326,7 +1346,7 @@ class ListBoxKeypressTest(unittest.TestCase):
 class PaddingTest(unittest.TestCase):
 	def ptest(self, desc, align, width, maxcol, left, right,min_width=None):
 		p = urwid.Padding(None, align, width, min_width)
-		l, r = p.padding_values((maxcol,))
+		l, r = p.padding_values((maxcol,),False)
 		assert (l,r)==(left,right), "%s expected %s but got %s"%(
 			desc, (left,right), (l,r))
 	
@@ -1777,9 +1797,33 @@ class CanvasOverlayTest(unittest.TestCase):
 		self.cotest("db3","\xA1\xA1\xA1\xA1\xA1\xA1",[],"OHIO",[],1,1,
 			" OHIO ",[(None,6)])
 
+class CanvasPadTrimTest(unittest.TestCase):
+	def cptest(self, desc, ct, ca, l, r, et, ea):
+		c = urwid.Canvas([ct], [ca])
+		c.pad_trim_left_right(l, r)
+		assert c.text[0] == et, "%s expected text %s, got %s"%(
+			desc, `et`,`c.text[0]`)
+		assert c.attr[0] == ea, "%s expected attr %s, got %s"%(
+			desc, `ea`,`c.attr[0]`)
+	
+	def test1(self):
+		self.cptest("none", "asdf", [], 0, 0, 
+			"asdf",[(None,4)])
+		self.cptest("left pad", "asdf", [], 2, 0, 
+			"  asdf",[(None,6)])
+		self.cptest("right pad", "asdf", [], 0, 2, 
+			"asdf  ",[(None,6)])
+	
+	def test2(self):
+		self.cptest("left trim", "asdf", [], -2, 0, 
+			"df",[(None,2)])
+		self.cptest("right trim", "asdf", [], 0, -2, 
+			"as",[(None,2)])
+
 def test_main():
 	for t in [
 		CalcWidthTest,
+		ConvertDecSpecialTest,
 		WithinDoubleByteTest,
 		CalcTextPosTest,
 		CalcBreaksCharTest,
@@ -1815,6 +1859,7 @@ def test_main():
 		SmoothBarGraphTest,
 		CanvasJoinTest,
 		CanvasOverlayTest,
+		CanvasPadTrimTest,
 		]:
 		if test_support.run_unittest(t): break
 	
