@@ -20,21 +20,111 @@
 #
 # Urwid web site: http://excess.org/urwid/
 
+from __future__ import nested_scopes
+
 from util import *
 from canvas import *
 from widget import *
-
-from __future__ import nested_scopes
+from escape import utf8decode
 
 try: True # old python?
 except: False, True = 0, 1
 
+class BigText(FixedWidget):
+	def __init__(self, markup, font):
+		"""
+		markup -- same as Text widget markup
+		font -- instance of a Font class
+		"""
+		self.set_font(font)
+		self.set_text(markup)
+	
+	def set_text(self, markup):
+		self.text, self.attrib = decompose_tagmarkup(markup)
+	
+	def get_text(self):
+		"""
+		Returns (text, attributes).
+		"""
+		return self.text, self.attrib
+	
+	def set_font(self, font):
+		self.font = font
+	
+	def pack(self, size=None, focus=False):
+		rows = self.font.height
+		cols = 0
+		for c in self.text:
+			cols += self.font.char_width(c)
+		return cols, rows
+	
+	def render(self, size, focus=False):
+		fixed_size(size) # complain if parameter is wrong
+		a = None
+		ai = ak = 0
+		o = []
+		rows = self.font.height
+		cols = 0
+		attrib = self.attrib+[(None,len(self.text))]
+		for ch in self.text:
+			if not ak:
+				a, ak = attrib[ai]
+				ai += 1
+			ak -= 1
+			width = self.font.char_width(ch)
+			if not width: 
+				# ignore invalid characters
+				continue
+			c = self.font.render(ch)
+			c.fill_attr(a)
+			o.append(c)
+			cols += width
+			o.append(width)
+		
+		if o:
+			return CanvasJoin(o[:-1])
+		return Canvas([""]*rows, maxcol=0)
+		
+
+class LineBox(WidgetWrap):
+	def __init__(self, w):
+		"""Draw a line around w."""
+		
+		tlcorner=None; tline=None; lline=None
+		trcorner=None; blcorner=None; rline=None
+		bline=None; brcorner=None
+		
+		def use_attr( a, t ):
+			if a is not None:
+				t = urwid.AttrWrap(t, a)
+			return t
+			
+		tline = use_attr( tline, Divider(utf8decode("─")))
+		bline = use_attr( bline, Divider(utf8decode("─")))
+		lline = use_attr( rline, SolidFill(utf8decode("│")))
+		rline = use_attr( rline, SolidFill(utf8decode("│")))
+		tlcorner = use_attr( tlcorner, Text(utf8decode("┌")))
+		trcorner = use_attr( trcorner, Text(utf8decode("┐")))
+		blcorner = use_attr( blcorner, Text(utf8decode("└")))
+		brcorner = use_attr( brcorner, Text(utf8decode("┘")))
+		top = Columns([ ('fixed', 1, tlcorner),
+			tline, ('fixed', 1, trcorner) ])
+		middle = Columns( [('fixed', 1, lline),
+			w, ('fixed', 1, rline)], box_columns = [0,2],
+			focus_column = 1)
+		bottom = Columns([ ('fixed', 1, blcorner),
+			bline, ('fixed', 1, brcorner) ])
+		pile = Pile([('flow',top),middle,('flow',bottom)],
+			focus_item = 1)
+		WidgetWrap.__init__(self, pile)
+
+
 class BarGraphError(Exception):
 	pass
 
 class BarGraph(BoxWidget):
-	eighths = u" ▁▂▃▄▅▆▇"
-	hlines =  u"_⎺⎻─⎼⎽"
+	eighths = utf8decode(" ▁▂▃▄▅▆▇")
+	hlines =  utf8decode("_⎺⎻─⎼⎽")
 	
 	def __init__(self, attlist, hatt=None, satt=None):
 		"""
@@ -622,7 +712,7 @@ def scale_bar_values( bar, top, maxrow ):
 
 
 class ProgressBar( FlowWidget ):
-	eighths = u" ▏▎▍▌▋▊▉"
+	eighths = utf8decode(" ▏▎▍▌▋▊▉")
 	def __init__(self, normal, complete, current=0, done=100, satt=None):
 		"""
 		normal -- attribute for uncomplete part of progress bar
